﻿using System;
using System.Collections.Generic;
using System.Text;
using Zcu.Mve.Core;
using Zcu.Mve.Visualization;
using Zcu.Mve.Numerics;
using Zcu.Mve.Strings;
using System.IO;
using System.ComponentModel;
using System.Windows.Forms.Design;


namespace Zcu.Mve.Compression
{
    /// <summary>
    /// Binary loader for dynamic meshes. Works with files previously written by the AnimationBinarySaver module.
    /// </summary>
    [ModuleInfo("Libor Váša", "Binary loader for dynamic meshes. Works with files previously written by the AnimationBinarySaver module.", CategoryType.Loader, CategoryData.MeshAnimated)]
    [PortInfo("Animation", "Loaded animation")]
    [PortInfo("FileName", "Name of the file to read. Port connection is not required, if nothing is connected then the module uses the FileName property")]
    class AnimationBinaryLoader : Module
    {
        public AnimationBinaryLoader()
        {
            AddOutPort("Animation", typeof(UniformDataArray));
            AddInPort("FileName", typeof(MveString), false);
        }

        string fileName = "animation.dat";

        /// <summary>
        /// Name of the file to read from. Only used when the FileName port is unconnected.
        /// </summary>        
        [Editor(typeof(System.Windows.Forms.Design.FileNameEditor), typeof(System.Drawing.Design.UITypeEditor))]
        [Description("Name of the file to read from. Only used when the FileName port is unconnected.")]
        public string FileName
        {
            get { return fileName; }
            set { fileName = value; }
        }

        UniformDataArray result = null;

        public override void Execute()
        {
            if (IsAllDataSame() && IsAllUserSettingsSame())
            {
                SetOutput("Animation", result, true);
            }
            else
            {
                if (GetInPort("FileName").Link != null)
                {
                    MveString fn = (MveString)GetInput("FileName");
                    this.fileName = fn.Value;
                }


                using (BinaryReader br = new BinaryReader(File.Open(fileName, FileMode.Open)))
                {

                    int triangleCount = br.ReadInt32();
                    UniformDataArray triangles = new UniformDataArray(typeof(Triangle), triangleCount);

                    for (int i = 0; i < triangles.Count; i++)
                    {
                        int v1 = br.ReadInt32();
                        int v2 = br.ReadInt32();
                        int v3 = br.ReadInt32();
                        triangles[i] = new Triangle(v1, v2, v3);
                    }

                    int frameCount = br.ReadInt32();
                    int pointCount = br.ReadInt32();

                    result = new UniformDataArray(typeof(TriangleMesh), frameCount);

                    for (int f = 0; f < frameCount; f++)
                    {
                        TriangleMesh frame = new TriangleMesh();
                        UniformDataArray points = new UniformDataArray(typeof(Point3D), pointCount);
                        for (int i = 0; i < pointCount; i++)
                        {
                            double x = br.ReadDouble();
                            double y = br.ReadDouble();
                            double z = br.ReadDouble();
                            points[i] = new Point3D(x, y, z);
                        }
                        UniformDataArray normals = new UniformDataArray(typeof(Normal3D), pointCount);
                        for (int i = 0; i < pointCount; i++)
                        {
                            double x = br.ReadDouble();
                            double y = br.ReadDouble();
                            double z = br.ReadDouble();
                            normals[i] = new Normal3D(x, y, z);
                        }
                        frame.Points = points;
                        frame.AddPointAttrs("normals", normals);
                        frame.AddCells("triangles", triangles);
                        result[f] = frame;
                    }
                }
                SetOutput("Animation", result);
            }
        }
    }
}
