﻿using System;
using System.Collections.Generic;
using System.Text;
using Zcu.Mve.Core;
using Zcu.Mve.Visualization;
using Zcu.Mve.Numerics;
using Zcu.Mve.Strings;
using System.IO;
using System.ComponentModel;
using System.Windows.Forms.Design;


namespace Zcu.Mve.Compression
{
    /// <summary>
    /// Binary saver fo dynamic meshes. Uses double precision, i.e. no data loss occurs.
    /// </summary>
    [ModuleInfo("Libor Váša", "Binary saver fo dynamic meshes. Uses double precision, i.e. no data loss occurs.", CategoryType.Saver, CategoryData.MeshAnimated)]
    [PortInfo("Animation", "Input animation to be saved into a file")]
    [PortInfo("FileName", "Name of the file to be saved to. Port connection is not required, if nothing is connected then the module uses the FileName property")]
    class AnimationBinarySaver:Module
    {
        public AnimationBinarySaver()
        {
            AddInPort("Animation", typeof(UniformDataArray));
            AddInPort("FileName", typeof(MveString), false);
        }

        string fileName = "animation.dat";

        /// <summary>
        /// Name of the file to save to. Only used when the FileName port is unconnected.
        /// </summary>
        [Description("Name of the file to save to. Only used when the FileName port is unconnected.")]
        public string FileName
        {
            get { return fileName; }
            set { fileName = value; }
        }

        double mult = 1;

        public double Mult
        {
            get { return mult; }
            set { mult = value; }
        }

        public override void Execute()
        {
            if (GetInPort("FileName").Link!=null)
            {
                MveString fn = (MveString)GetInput("FileName");
                this.fileName = fn.Value;
            }

            using (BinaryWriter bw = new BinaryWriter(new FileStream(fileName, FileMode.Create)))
            {
                UniformDataArray input = (UniformDataArray)GetInput("Animation");
                UnstrGrid firstFrame = (UnstrGrid)input[0];
                UniformDataArray triangles = firstFrame.GetCells(typeof(Triangle))[0];

                bw.Write(triangles.Count);
                for (int i = 0; i < triangles.Count; i++)
                {
                    Triangle t = (Triangle)triangles[i];
                    bw.Write(t.V1);
                    bw.Write(t.V2);
                    bw.Write(t.V3);
                }
                int pointCount = firstFrame.Points.Count;
                bw.Write(input.Count);
                bw.Write(pointCount);

                double max = 0;
                // find max
                for (int f = 0; f < input.Count; f++)
                {
                    UnstrGrid frame = (UnstrGrid)input[f];
                    for (int i = 0; i < pointCount; i++)
                    {
                        Point3D p = (Point3D)frame.Points[i];
                        if (Math.Abs(p.X) > max)
                            max = Math.Abs(p.X);
                        if (Math.Abs(p.Y) > max)
                            max = Math.Abs(p.Y);
                        if (Math.Abs(p.Z) > max)
                            max = Math.Abs(p.Z);
                    }
                }

                mult = 100 / max;

                for (int f = 0; f < input.Count; f++)
                {
                    UnstrGrid frame = (UnstrGrid)input[f];
                    UniformDataArray normals = frame.GetPointAttrs(typeof(Normal3D))[0];
                    for (int i = 0; i < pointCount; i++)
                    {
                        Point3D p = (Point3D)frame.Points[i];
                        bw.Write((float)(p.X * mult));
                        bw.Write((float)(p.Y * mult));
                        bw.Write((float)(p.Z * mult));
                        Normal3D n = (Normal3D)normals[i];
                        bw.Write((float)n.Nx);
                        bw.Write((float)n.Ny);
                        bw.Write((float)n.Nz);
                    }
                }
            }
        }
    }
}
