﻿using System;
namespace Zcu.Mve.Compression
{
    /// <summary>
    /// Intergace for scalar quantizers
    /// </summary>
    public interface IQuantizer
    {
        /// <summary>
        /// Serializes the internal structures of the quantizer onto a given stream <paramref name="s"/>
        /// </summary>
        /// <param name="s">Stream to serialize into</param>
        void Serialize(System.IO.Stream s);

        /// <summary>
        /// Restores internal structures of the quantizer from a given stream <paramref name="s"/>, previously serialized by a call to Serialize
        /// </summary>
        /// <param name="s">Stream to read from</param>
        void Deserialize(System.IO.Stream s);
        
        /// <summary>
        /// Quantizes a scalar value into an integer
        /// </summary>
        /// <param name="v">Value to quantize</param>
        /// <returns>quantized value</returns>
        int Quantize(double v);

        /// <summary>
        /// Restores a value from integer
        /// </summary>
        /// <param name="q">Quantized integer input</param>
        /// <returns>Restored value</returns>
        double Restore(int q);
    }
}
